-- Using AceAddon-3.0
-- Creating an addon object
-- Including AceConsole functionality; "AceConsole-3.0"
-- Including AceEvent functionality; "AceEvent-3.0"
-- Including AceHook functionality; "AceHook-3.0"
ExPack = LibStub("AceAddon-3.0"):NewAddon("ExPack", "AceEvent-3.0", "AceConsole-3.0", "AceHook-3.0");
local myAddonName = "ExPack";

-- Using translations
local L = LibStub("AceLocale-3.0"):GetLocale(myAddonName, "enUS", true);
-- Registering the options
local AceConfig = LibStub("AceConfigDialog-3.0");

ExPack.guildBank = {}
ExPack._bagUpdateDebounce = nil
ExPack.version = "v0.4.0";
ExPack.isMerchantOpen = false
ExPack.profile = {}

local WoWRetail = (WOW_PROJECT_ID == WOW_PROJECT_MAINLINE)
local WoWClassic = (WOW_PROJECT_ID ~= WOW_PROJECT_MAINLINE)

local hooks = {}
function ExPack:RegisterAddonHook(addon, callback)
	if IsAddOnLoaded(addon) then
		callback()
	else
		hooks[addon] = callback
	end
end

-- key bindings
BINDING_HEADER_EXPACK = L["Item Expansion Pack"]
BINDING_NAME_EXPACK_CAPTION = L["Expack Show/Hide Caption"]
BINDING_NAME_EXPACK_GOLDENCROWN = L["Expack Show/Hide Goldencrown"]
BINDING_NAME_EXPACK_BANK_ONLY = L["Expack Toggle Banks Only filter"]
BINDING_NAME_EXPACK_ANY_ITEM = L["Expack Toggle Any Item filter"]
BINDING_NAME_EXPACK_ANY_EXPAN = L["Expack Toggle Any Expansion filter"]

function ExPack:ADDON_LOADED(event, addon)
	self:dPrint("ADDON_LOADED; "..addon)
	if hooks[addon] then
		hooks[addon]()
		hooks[addon] = nil
	end
end

function ExPack:OnInitialize()
	-- Code that you want to run when the addon is first loaded goes here.
	--ExPack.logger:Trace("Begin ExPack")

	self.iCount = 5

	self.config = self:GetModule("Config")
	self.logger = self:GetModule("Logger")
	self.utils = self:GetModule("Utils")

	self.config:Init(self)
	self.logger:Init(self.config)

	-- Using AceConsole for slash commands
	--self:RegisterChatCommand( myAddonName, "SlashCommand" );
	--self:dPrint("AceConsole")
	self:RegisterChatCommand( "ep", "ChatCommand" );

	--Using AceDB-3.0
	-- SavedVariables already prepared in toc file
	-- Initializing AceDB
	-- CVAR_UPDATE "autoLootDefaut", "1"
	--self:dPrint("AceDB")

	if not Ghordian_ExPackDB then
		Ghordian_ExPackDB = {}
	end

	--self.db = LibStub("AceDB-3.0"):New("Ghordian_ExPackDB", ExPack.defaults)
	-- Working with profiles
	--if (self.db.profile == nil) or not (self.db.profile) then 
		--self.db.profile = ExPack.defaults.profile; 
		--ExPack.profile = self.db.profile;
	--end;

	self.profile = self.config.db.profile;

	self:LoadPlugins();

	--[[ Registering the options REMOVED V3.4
	--self:Print("AceConfig")
	LibStub("AceConfig-3.0"):RegisterOptionsTable(myAddonName, ExPack.options);
	AceConfig:AddToBlizOptions(myAddonName, myAddonName);
	]]--

	-- Standard hooking
	--self:dPrint("AceHook")
	--[[ SIMPLIFIED BY ContainerFrame_GenerateFrame
	self:SecureHook("OpenAllBags", "ExPack_OpenAllBags");
	self:SecureHook("ToggleAllBags", "ExPack_ToggleAllBags");
	self:SecureHook("ToggleBag", "ExPack_ToggleBag");
	self:SecureHook("OpenBag", "ExPack_OpenBag");
	]]--
	--[[ Events:
		BAG_UPDATE
		BAG_UPDATE_DELAYED
	--
		https://wowpedia.fandom.com/wiki/API_C_Item.RequestLoadItemData
		C_Item.RequestLoadItemData(itemLocation)
			RequestLoadItemData
			itemLocation
			ItemLocationMixin

		C_Item.RequestLoadItemDataByID(itemInfo)
			RequestLoadItemDataByID
		itemInfo
			string : ItemLink or ID

		https://wowpedia.fandom.com/wiki/ITEM_DATA_LOAD_RESULT
		ITEM_DATA_LOAD_RESULT: itemID, success

		ItemButton.UpdateItemContextMatching

		--self:SecureHook(C_Container, "PickupContainerItem", "ExPack_PickupContainerItem");
	--]]

	--C_Container.SetItemSearch(searchString)
	--TODO : Filtering by expansion name/abbreviation
	--self:SecureHook(C_Container, "SetItemSearch", "ExPack_SetItemSearch");

	self:SecureHook("BankFrameItemButton_Update", "ExPack_BankFrameItemButton_Update");

--[[ New mechanism using Plugin_Default --]]
	if (self.PluginsMode == false) then
		if _G.ContainerFrame_Update then
			self:SecureHook("ContainerFrame_Update", "ExPack_ContainerFrame_Update");
			self.logger:Info("ContainerFrame_Update");
		else
			--DragonFlight improvement
			self:SecureHook("ContainerFrame_GenerateFrame", ExPack_ContainerFrame_GenerateFrame)
		end
	end

	self:SecureHook("MerchantFrame_Update", ExPack_MerchantFrame_Update)

	--self:SecureHook("OpenMail_GetItemCounts", ExPack_OpenMail_GetItemCounts);
	self:SecureHook("OpenMailFrame_UpdateButtonPositions", ExPack_OpenMailFrame_UpdateButtonPositions);
	self:SecureHook("InboxFrame_Update", ExPack_InboxFrame_Update);
	self:SecureHook("SendMailFrame_Update", ExPack_SendMailFrame_Update);

	--self:dPrint("AceEvent")
	if (self:CheckDB() == true) then
		-- Including AceEvent functionality
		self:RegEvents();
	end;

	-- Using AceConsole for output; used as a mixin for AceAddon
	ExPack.logger:Info("ExPack by |cff69CCF0Ghordian");
end

function ExPack:OnEnable()
	self.logger:Debug("OnEnable |cff00CC00LOADED");
	-- Called when the addon is enabled
	self:RegEvents();
	--
	--self:CreateConfigPanels();
end

function ExPack:OnDisable()
	self.logger:Debug("OnDisable |cff00CC00LOADED");
	-- Called when the addon is disabled
	self:UnRegEvents();
	-- todo - finally cleanup containers
end

function ExPack:ChatCommand(input)
	-- Process the slash command ('input' contains whatever follows the slash command)
	if (input == "minimap") then
		self:ToggleMinimapButton()
	else
		self:OpenOptions()
	end
end;

function ExPack:OpenOptions()
	-- first time opens Interface Frame
	InterfaceOptionsFrame_OpenToCategory(myAddonName);
	-- Second time opens Category Option Panel
	InterfaceOptionsFrame_OpenToCategory(myAddonName);
end;

function ExPack:CheckDB()
	if self.config:UseLibItemInfo() then 	
		local LII = LibStub("LibItemInfo-1.0");
		if LII == nil then
			self.logger:Error("LibItemInfo |cffCC0000NOT LOADED");
		else
			self.logger:Info("LibItemInfo |cff00CC00LOADED");
		end
		return false;
	elseif self.config:UseItemVersion() then
		if not (ExPack.itemIdToVersionId) then return false; end;
		if not (ExPack.versionIdToVersion) then return false; end;
		self.logger:Info("ItemVersion |cffCC0000NOT LOADED");
	end
	return true;
end;

function ExPack:RegEvents()
	-- Subscribing to events
	--self:dPrint("RegEvents")
	self:RegisterMessage("EXPACK_CONFIG_CHANGED", "CONFIG_CHANGED")

	--self:RegisterEvent("ADDON_LOADED")

	-- Events for Guild Bank UI Dragonflight 10.0.02
	--REMOVED self:RegisterEvent("GUILDBANKFRAME_OPENED")
	--REMOVED self:RegisterEvent("GUILDBANKFRAME_CLOSED") 
	self:RegisterEvent("GUILDBANKBAGSLOTS_CHANGED")

	-- Events for Player bank UI
	self:RegisterEvent("BANKFRAME_OPENED")
	self:RegisterEvent("PLAYERREAGENTBANKSLOTS_CHANGED")

	self:RegisterEvent("BAG_UPDATE");
	self:RegisterEvent("BAG_NEW_ITEMS_UPDATED");
	self:RegisterEvent("BAG_UPDATE_DELAYED");
	self:RegisterEvent("ITEM_PUSH");

	self:RegisterEvent("MERCHANT_SHOW");
	self:RegisterEvent("MERCHANT_CLOSED");
	self:RegisterEvent("MERCHANT_FILTER_ITEM_UPDATE");
	self:RegisterEvent("MERCHANT_UPDATE");

	self:RegisterEvent("MAIL_INBOX_UPDATE");
	self:RegisterEvent("MAIL_SEND_INFO_UPDATE");

	-- https://wowpedia.fandom.com/wiki/Events - /etrace

end

function ExPack:UnRegEvents()

	-- todo - clear previous ExPack info for all the containers

	-- Events for Guild Bank UI Dragonflight 10.0.02
	--REMOVED self:UnregisterEvent("GUILDBANKFRAME_OPENED")
	self:UnregisterEvent("GUILDBANKBAGSLOTS_CHANGED")

	self:UnregisterEvent("BAG_UPDATE");
	self:UnregisterEvent("BAG_UPDATE_DELAYED");
	self:UnregisterEvent("BAG_NEW_ITEMS_UPDATED");
	self:UnregisterEvent("ITEM_PUSH");

	self:UnregisterEvent("MERCHANT_SHOW");
	self:UnregisterEvent("MERCHANT_CLOSED");
	self:UnregisterEvent("MERCHANT_FILTER_ITEM_UPDATE");
	self:UnregisterEvent("MERCHANT_UPDATE");

	-- Events for Player bank UI
	self:UnregisterEvent("BANKFRAME_OPENED")
	self:UnregisterEvent("PLAYERREAGENTBANKSLOTS_CHANGED")

end

function ExPack:CONFIG_CHANGED()
	-- todo - clear previous ExPack info for all the containers

	self:UpdateSlots()
end

--[[
function ExPack:GUILDBANKFRAME_OPENED()
	--if not self.profile.Enabled then return end
	ExPack:dPrint("OnGuildBankFrameOpened")

	ExPack.guildBank.isGuildBankOpen = true
	ExPack.guildBank.numTabs = GetNumGuildBankTabs()
	ExPack.guildBank.guildFunds = GetGuildBankMoney()
	ExPack.guildBank.tab = GetCurrentGuildBankTab()

	--register the _CLOSED event whenever the bank is opened
	--REMOVED ExPack:RegisterEvent("GUILDBANKFRAME_CLOSED") 
	--REDO ExPack:RegisterEvent("GUILDBANKBAGSLOTS_CHANGED")
end

function ExPack:GUILDBANKFRAME_CLOSED() 
	--if not self.profile.Enabled then return end
	ExPack:dPrint("OnGuildBankFrameClosed")

	--REMOVED ExPack:UnregisterEvent("GUILDBANKFRAME_CLOSED") 
	ExPack:UnregisterEvent("GUILDBANKBAGSLOTS_CHANGED")

	ExPack:GuildBankFrame_RemoveExPack();
end
]]--

function ExPack:GUILDBANKBAGSLOTS_CHANGED() 
	ExPack.guildBank.isGuildBankOpen = true
	--if not self.profile.Enabled then return end
	--ExPack:dPrint("GUILDBANKBAGSLOTS_CHANGED");
	ExPack_Debounce(ExPack_OnGuildBankFrameChanged, 0.01)
end

--[[
	BagnonGuildFrame1 name found thanks to 
	DEBUGGING; scanned via /tinspect GlobalName
	/tinspect ElvUI_ContainerFrameBag5Slot32 
]]--

function ExPack_OnGuildBankFrameChanged() 
	if not ExPack.profile.Enabled then return end
	if ExPack.guildBank.isGuildBankVisited == false then
		ExPack.guildBank.isGuildBankVisited = true
	end
	ExPack.logger:Trace("OnGuildBankFrameChanged")
	local num, itemString
	if Bagnon or BagBrother then
		--ExPack:dPrint("BagnonGuildFrame1; ")
		ExPack:vardump(BagnonGuildFrame1)
	end
	if (GuildBankFrame and GuildBankFrame:IsVisible())
		or (BagnonGuildFrame1 and BagnonGuildFrame1:IsVisible()) then
		local tab = GetCurrentGuildBankTab();
		if ( ExPack.guildBank.tab ~= tab ) then
			if (tab ~= nil) and (ExPack.guildBank.tab ~= nil) then
				--ExPack:jPrint("GuildBankFrame.Tab differ; "..tab.."; "..ExPack.guildBank.tab);
			end
			ExPack:GuildBankFrame_RemoveExPack();
		end
		ExPack.guildBank.tab = tab;

		-- GuildBankFrame_OnLoad(..)
		-- GuildBankFrame.mode == "bank"
		local button, index, column;
		local NumSlotsPerGuildBankGroup = 14 --NUM_SLOTS_PER_GUILDBANK_GROUP;
		local btnCount = 0
		local thisContainer = ContainerTypes[ExPack.Types.GUILDBANK]
		local size = thisContainer:GetSize();
		--ExPack:dPrint("GuildBankSize; "..size.."; "..NumSlotsPerGuildBankGroup)
		for slot=1, size do
			index = mod(slot, NumSlotsPerGuildBankGroup);
			if ( index == 0 ) then
				index = NumSlotsPerGuildBankGroup;
			end
			column = ceil((slot-0.5) / NumSlotsPerGuildBankGroup);
			--ExPack:jPrint("GuildBankColumn; tab; "..tab.."; slot; "..slot.."; column"..column.."; index; "..index);
			button = _G["GuildBankColumn"..column.."Button"..index];
			if button == nil then
			--	ExPack:dPrint("GuildBankColumn fail")
				if (GuildBankFrame) then
					button = GuildBankFrame.Columns[column].Buttons[index];
				elseif (BagnonGuildFrame1) then
					-- todo button = BagnonGuildFrame1.Buttons[index];
				end
			end
			if button ~= nil then
				ExPack:ScanBagFrame(tab, slot, button, thisContainer);
				btnCount = btnCount + 1
			else
				--ExPack:dPrint("GuildBankFrame fail")
			end
		end
		if ( btnCount < size ) then
			ExPack:jPrint("GuildBankColumn; "..btnCount.."; "..size);
		end
		ExPack:dReset();
	else
		--ExPack:dPrint("No GuildBankFrame; ");
	end
	ExPack.guildBank.isGuildBankOpen = false
end

function ExPack:BANKFRAME_OPENED()
	if not self.profile.Enabled then return end
	ExPack.logger:Debug("OnBankFrameOpened");

	local frame = BankFrame;
	if (frame.activeTabIndex == 1) then
		for bagID = NUM_BAG_SLOTS + 1, NUM_BAG_SLOTS + NUM_BANKBAGSLOTS do		-- 5 to 11
			ExPack:ScanBag(bagID)
		end
		--ExPack:ScanBank();
	elseif (frame.activeTabIndex == 2) then
		ExPack:ScanReagentBank();
	end

	ExPack:RegisterEvent("BANKFRAME_CLOSED")
	ExPack:RegisterEvent("PLAYERBANKSLOTS_CHANGED")
	ExPack:RegisterEvent("REAGENTBANK_UPDATE")
	ExPack:RegisterEvent("PLAYERREAGENTBANKSLOTS_CHANGED")
end

function ExPack:BANKFRAME_CLOSED()
	if not self.profile.Enabled then return end
	--
	ExPack.logger:Debug("OnBankFrameClosed")

	ExPack:UnregisterEvent("BANKFRAME_CLOSED")
	ExPack:UnregisterEvent("PLAYERBANKSLOTS_CHANGED")

	-- done ; clean after close
	ExPack:BankFrame_RemoveExPack();
end

function ExPack:PLAYERBANKSLOTS_CHANGED(slot)
	if not self.profile.Enabled then return end
	--
	--ExPack.logger:Trace("OnPlayerBankSlotsChanged; "..slot)
	--ExPack:dReset()

	ExPack:ScanContainer(BANK_CONTAINER, ExPack.Types.BANK)
end

function ExPack:REAGENTBANK_UPDATE()
	if not self.profile.Enabled then return end
	--ExPack.logger:Trace("ReagentbankUpdate")

	ExPack:ScanReagentBank()
end

function ExPack:PLAYERREAGENTBANKSLOTS_CHANGED(slot)
	if not self.profile.Enabled then return end
	--ExPack.logger:Trace("OnReagentBankFrameChanged; "..slot)

	ExPack:ScanReagentBank()
end

function ExPack:MERCHANT_SHOW()
	self.logger:Debug("MERCHANT_SHOW; ")
	self.isMerchantOpen = true
end	

function ExPack:MERCHANT_CLOSED()
	self.logger:Debug("MERCHANT_CLOSED; ")
	if self.isMerchantOpen then
		self.isMerchantOpen = false
		ExPack_ResetVendorPage("close")
	end
end

function ExPack:MERCHANT_FILTER_ITEM_UPDATE()
	ExPack.logger:Debug("MERCHANT_FILTER_ITEM_UPDATE; ")
	if self.isMerchantOpen then
		ExPack_MerchantFrame_Update()
	end
end

function ExPack:MERCHANT_UPDATE()
	ExPack.logger:Debug("MERCHANT_UPDATE; ")
	if self.isMerchantOpen then
		ExPack_MerchantFrame_Update()
	end
end

function ExPack_ResetVendorPage(info)
	ExPack.logger:Debug("ExPack_ResetVendorPage; "..info)

	for i = 1, MERCHANT_ITEMS_PER_PAGE, 1 do
		local btnName = "MerchantItem" .. i .. "ItemButton";
		local btnIconName = btnName..myAddonName;
		local itemButton = _G[btnName]
		if (itemButton) then
			if (itemButton.ExPack) then
				itemButton.ExPack:Hide()
				itemButton.ExPack = null
			end
		end

		local tframe = _G[btnIconName]
		if (tframe) then
			tframe:Hide()
			_G[btnIconName] = null
		end
	end
end

function ExPack_MerchantFrame_Update()

	if ((ExPack.isMerchantOpen == false) or (ExPack.profile.Enabled == false)) then
		--ExPack.logger:Warn("MerchantFrame_Update; bagupdate/loot/mail?")
		return;
	end

	ExPack.logger:Debug("ExPack_MerchantFrame_Update; "..MerchantFrame.page)
	ExPack_ResetVendorPage("update; page;"..MerchantFrame.page);

	local profile = ExPack.profile
	ExPack.iCount = 5

	local bClear = false
	if not profile.filtersShowMerchants then
		bClear = true
	end

	for i = 1, MERCHANT_ITEMS_PER_PAGE, 1 do
		local index = (((MerchantFrame.page - 1) * MERCHANT_ITEMS_PER_PAGE) + i);
		local itemLink = GetMerchantItemLink(index);
		local itemID = nil
		if (itemLink) then
			--ExPack:dPrint("MerchantFrame_Update; ItemLink found; "..index)
			--SharedXML\LinkUtil
			itemID = GetItemInfoFromHyperlink(itemLink)
			if (itemID) then
				--ExPack:dPrint("MerchantFrame_Update; ItemID found; "..itemID)
			end
		end

		local btnName = "MerchantItem" .. i .. "ItemButton";
		local itemButton = _G[btnName]

		if (itemID) then
			ExPack_GetExPackFrame(itemButton, itemID, itemLink, 0, 0, bClear);
			ExPack_UpdateExPackFrame(itemButton, bCleanUp);
		else
			-- clean up
		end -- itemID

	end -- for
end

function ExPack_GetExpackInfoFromItemID(itemID , expacID)

	local expname = "n/a";
	local clr = "|cFFffff00";

	if (itemID > 0) and (expacID == 0) then
		local itemVersion, itemExpac = ExPack:myGetVersion(itemID);
		if itemVersion then
			--ExPack:vardump (itemVersion);
			itemExpac = ExPack:myGetVersionExpac(itemVersion);
			-- if itemExpac ~= nil then
				-- expname = itemExpac.shortName;
				-- clr = itemExpac.color;
				-- expacID = itemVersion.major;
			-- else
				-- ExPack:dPrint("itemExpac nil; "..itemID.." ; "..quality)
				-- if not self.profile.UseItemVersion then
					-- don't show "n/a"
					-- expname = "";
				-- end
			-- end
		end

		if itemExpac then
			if itemExpac ~= nil then
				expname = itemExpac.shortName;
				clr = itemExpac.color;
				expacID = itemExpac.expansionID;
			else
				ExPack.logger:Debug("itemExpac nil; "..itemID.." ; "..quality)
				if not self.profile.UseItemVersion then
					-- don't show "n/a"
					expname = "";
				end
			end
		else
			quality = quality or 0;
			--ExPack:dPrint("itemVersion nil; "..itemID.." ; "..quality)
			if not ExPack.profile.UseItemVersion then
				-- don't show "n/a"
				expname = "";
			end
		end
	end

	return expname, clr, expacID;
end

function ExPack_GetColor(clr)
	local Color = {}
	local r,g,b
	r, g, b = clr:match("|cFF(%x%x)(%x%x)(%x%x)")
	r, g, b = tonumber(r, 16), tonumber(g, 16), tonumber(b, 16)
	Color.r, Color.g, Color.b = r / 255, g / 255, b / 255
	return Color
end

function ExPack_UpdateExPackFrame(itemButton, bCleanUp)

	local profile = ExPack.profile

	if (itemButton.ExPackIcon) then
		if (profile.visualShowCrown and not bCleanUp) then
			--ExPack.logger:Debug("ExPack_UpdateExPackFrame; ShowCrown; "..itemID)
			itemButton.ExPackIcon:Show()
		else
			--ExPack.logger:Debug("ExPack_UpdateExPackFrame; HideCrown; ")
			itemButton.ExPackIcon:Hide()
			if (bCleanUp) then
				itemButton.ExPackIcon = nil;
			end
		end
		local itemFrameName = itemButton:GetName()
		local itemIconName = itemFrameName .. "ExPack"
		_G[itemIconName] = nil;
	end

	if (itemButton.ExPack) then
		if profile.visualShowCaption and not bCleanUp then
			--ExPack.logger:Debug("ExPack_UpdateExPackFrame; ShowCaption; "..itemID)
			itemButton.ExPack:Show()
		else
			--ExPack.logger:Debug("ExPack_UpdateExPackFrame; HideCaption; ")
			itemButton.ExPack:Hide()
			if (bCleanUp) then
				itemButton.ExPack = nil;
			end
		end
	end

end

function ExPack_GetExPackFrame(bagItemButton, itemID, itemLink, bankCol, bankIdx, bClear)
	if (bagItemButton == nil) then
		return
	end
	local bagItemButtonName = bagItemButton:GetName()
	if (bagItemButtonName == nil) then
		bagItemButtonName = "GuildBankItemButtonC" .. bankCol .. "B" .. bankIdx
	end

	local btnName = bagItemButtonName .. myAddonName;
	local tframe = _G[btnName]

	if (not itemID) then
		return tframe;
	end
	if (not itemLink) then
		return tframe;
	end

	local expname, clr, expacID  = ExPack_GetExpackInfoFromItemID(itemID, 0);
	local bCanShow = ExPack:CanShowItemLink(itemLink, expacID) and not bClear;

	local Color = ExPack_GetColor(clr);

	local profile = ExPack.profile

	if bCanShow and (itemID > 0) then
		if profile.visualShowCrown then
			if (tframe == nil) then
				-- ShowBorder/Golden Crown
				tframe = CreateFrame("Frame", btnName, bagItemButton, "BackdropTemplate")
				tframe:ClearAllPoints()
				local crownSize = profile.visualCrownSize;
				tframe:SetSize(crownSize, crownSize)
				local crownPosition = profile.visualCrownPosition;
				local crownOffsetX = profile.visualOffsetX;
				local crownOffsetY = profile.visualOffsetY;
				tframe:SetPoint(crownPosition, bagItemButton, crownPosition, crownOffsetX, crownOffsetY)
				if ExPack.iCount > 0 then
				--ExPack.logger:Debug("ExPack_GetExPackFrame; ShowCrown; "..ExPack.iCount.."; x; "..crownOffsetX.."; y; "..crownOffsetY)
					ExPack.iCount = ExPack.iCount - 1
				end
			end
			tframe.icon = tframe:CreateTexture(nil, "OVERLAY")
			tframe.icon:SetTexture("Interface/Addons/Ghordian_ExPack/images/GoldenCrown.tga")
			tframe.icon:SetAllPoints(tframe)
			tframe.icon:SetBlendMode("BLEND")
			tframe.icon:SetPoint("CENTER", tframe)
			if (itemID > 0) then
				tframe.icon:SetVertexColor(Color.r, Color.g, Color.b, 1)
				--tframe:Show();
			--ExPack.logger:Trace("ExPack_GetExPackFrame; ShowCrown; "..itemID)
			else
				tframe:Hide();
			--ExPack:dPrint("ExPack_GetExPackFrame; No ItemID; "..itemID)
			end
			bagItemButton.ExPackIcon = tframe;
		else
		--ExPack.logger:Trace("ExPack_GetExPackFrame; visualShowCrown; "..itemID)
		end

		if profile.visualShowCaption then
			if not bagItemButton.ExPack then
				bagItemButton.ExPack = bagItemButton:CreateFontString(nil, "OVERLAY", "NumberFontNormalSmall")
				bagItemButton.ExPack:SetPoint("TOPLEFT", tframe, 0, -2)
				bagItemButton.ExPack:SetTextColor(1,1,0) -- yellow!
				bagItemButton.ExPack:SetText("")
			--ExPack.logger:Trace("ExPack_GetExPackFrame; ExPack; created; "..itemID)
			end

			-- Apply appearance configuration
			bagItemButton.ExPack:ClearAllPoints()
			bagItemButton.ExPack:SetPoint(profile.visualPosition, unpack(ExPack.config.PositionOffsets[profile.visualPosition]))
			bagItemButton.ExPack:SetFontObject(ExPack.config.Fonts[profile.visualFont] or NumberFontNormal)
			bagItemButton.ExPack:SetTextColor(Color.r, Color.g, Color.b)
			bagItemButton.ExPack:SetText( expname )
			bagItemButton.ExPack:SetScale( profile.visualScale )
			bagItemButton.ExPack:Hide()
		--ExPack.logger:Trace("ExPack_GetExPackFrame; ExPack; set; "..expname)
		else
		--ExPack.logger:Trace("ExPack_GetExPackFrame; visualShowCaption; "..itemID)
		end

	else
		if (tframe) then
			tframe:Hide();
			--ExPack.logger:Trace("ExPack_GetExPackFrame; tframe; hide")
		end
		if bagItemButton.ExPack then
			bagItemButton.ExPack:Hide();
			--ExPack.logger:Trace("ExPack_GetExPackFrame; bagItemButton.ExPack; hide")
		end
	end

	return tframe
end


-- https://wowpedia.fandom.com/wiki/ITEM_PUSH
-- ITEM_PUSH: bagSlot, iconFileID
function ExPack:ITEM_PUSH(event, bagSlot, iconFileID)
	self.logger:Debug("ExPack; "..event.."; bagSlot; "..bagSlot)
end

local bag_update_time = 0;
local lastBagID = -1;

--https://wowpedia.fandom.com/wiki/BAG_NEW_ITEMS_UPDATED
function ExPack:BAG_NEW_ITEMS_UPDATED(event)
--	bag_update_time = GetTime() - bag_update_time
	self.logger:Debug("ExPack; "..event.."; lastBagID; "..lastBagID)
	--if (bag_update_time > 1) then
		ExPack:ExPack_BagsUpdate(event, lastBagID);
	--end
--	bag_update_time = GetTime();
end

--https://wowpedia.fandom.com/wiki/BAG_UPDATE_DELAYED
function ExPack:BAG_UPDATE_DELAYED(event)
	bag_update_time = GetTime() - bag_update_time
	self.logger:Debug("ExPack; "..event.."; "..bag_update_time.."; lastBagID; "..lastBagID)
	--if not self.profile.Enabled then return end
	--
	ExPack:ExPack_BagsUpdate(event, lastBagID);
	--lastBagID = -1;
	bag_update_time = GetTime();
end

--https://wowpedia.fandom.com/wiki/BAG_UPDATE
--	BAG_UPDATE: bagID
function ExPack:BAG_UPDATE(event, bagID)
	bag_update_time = GetTime() - bag_update_time
	self.logger:Debug("ExPack; "..event.."; "..bag_update_time.."; "..bagID)
	--if not self.profile.Enabled then return end
	--
	--if (bag_update_time > 1) then
		ExPack:ExPack_BagsUpdate(event, bagID);
		lastBagID = bagID
	--end
	bag_update_time = GetTime();
end

function ExPack_OnEvent(self, event, ...)

	ExPack:dPrint("ExPack_OnEvent; "..event);

end

--https://wowpedia.fandom.com/wiki/INVENTORY_SEARCH_UPDATE
function ExPack:INVENTORY_SEARCH_UPDATE(event, ...)
--[[
	ExPack:dPrint("ExPack; "..event);
--]]
end

function ExPack_Debounce(fn, seconds)
	if (ExPack._bagUpdateDebounce) then
		ExPack._bagUpdateDebounce.cancelled = true
	end

	local timer = {
		func = fn,
		cancelled = false,
	}

	timer.callback = function()
		if not timer.cancelled then
			timer.func()
			ExPack._bagUpdateDebounce = nil
		end
	end

	ExPack._bagUpdateDebounce = timer
	C_Timer.After(seconds, timer.callback)
	gTimer = timer
end

--[[

function ExPack_OnLoad(self, event)
	ExPack:dPrint("ExPack_OnLoad; ");
end

function ExPack_ExecuteButtonUpdates()
	ExPack:dPrint("ExPack_ExecuteButtonUpdates; ");
end

function ExPack_ExecuteButtonUpdatesGuildBank()
	ExPack:dPrint("ExPack_ExecuteButtonUpdatesGuildBank; ");
end

function ExPack_CloseGuildBankFrame()
	ExPack:dPrint("ExPack_CloseGuildBankFrame; ");
	ExPack:BANKFRAME_CLOSED()
end

--]]

--[[
	key bindings
--]]
function ExPack:ToggleFrame()
	ExPack:dPrint(BINDING_HEADER_EXPACK);
	ExPack:OpenOptions()
end

function ExPack:ToggleVisualCaption()
	ExPack:dPrint(BINDING_NAME_EXPACK_CAPTION);
	ExPack.profile.visualShowCaption = not ExPack.profile.visualShowCaption;
	ExPack:ExPack_ScanBags();
end

function ExPack:ToggleFilterBanksOnly()
	ExPack:dPrint(BINDING_NAME_EXPACK_BANK_ONLY);
	ExPack.profile.filtersShowBanks = not ExPack.profile.filtersShowBanks;
	ExPack:ExPack_ScanBags();
end

function ExPack:ToggleFilterAnyItem()
	ExPack:dPrint(BINDING_NAME_EXPACK_ANY_ITEM);
	ExPack.profile.filtersShowAll = not ExPack.profile.filtersShowAll;
	ExPack:ExPack_ScanBags();
end

function ExPack:ToggleFilterAnyExpansion()
	ExPack:dPrint(BINDING_NAME_EXPACK_ANY_EXPAN);
	ExPack.profile.filtersAnyExpansion = not ExPack.profile.filtersAnyExpansion;
	ExPack:ExPack_ScanBags();
end

--[[

--	Mail Attachments support

function ExPack_GetInboxItem(messageIndex, attachmentIndex)
	-- GetInboxItem(InboxFrame.openMailID, i);
	ExPack:dPrint( "GetInboxItem; "..messageIndex..";"..attachmentIndex );
end

--]]

function ExPack_InboxFrame_Update()
	ExPack:dPrint( "InboxFrame_Update; " );

	local profile = ExPack.profile

	local bCleanUp = false
	if not profile.filtersShowOpenMailAttachments then
		--ExPack.logger:Warn("InboxFrame_Update; filtersShowOpenMailAttachments; false")
		bCleanUp = true
	end

	local numItems, totalItems = GetInboxNumItems();

	local index = ((InboxFrame.pageNum - 1) * INBOXITEMS_TO_DISPLAY) + 1;
	for i=1, INBOXITEMS_TO_DISPLAY do
		if ( index <= numItems ) then
			-- Setup first button itemLink
			packageIcon, stationeryIcon, sender, subject, money, CODAmount, daysLeft, itemCount, wasRead, x, y, z, isGM, firstItemQuantity, firstItemLink = GetInboxHeaderInfo(index);
			button = _G["MailItem"..i.."Button"];
			if (button) then
				local tframe = nil;
				local itemID = 0;
				if (firstItemLink) then
					itemID = GetItemInfoFromHyperlink(firstItemLink)
					tframe = ExPack_GetExPackFrame(button, itemID, firstItemLink, 0, 0, bCleanUp);
				else
					--ExPack:dPrint("InboxFrame_Update; firstItemLink; not found; "..index)
				end
				if (itemID) then
					if (tframe) then
						if profile.visualShowCrown and not bCleanUp then
							--ExPack:dPrint("InboxFrame_Update; tframe; ShowCrown; "..index.."; "..itemID)
							tframe:Show()
						else
							--ExPack:dPrint("InboxFrame_Update; tframe; HideCrown; "..index.."; "..itemID)
							tframe:Hide()
							if (bCleanUp) then
								tframe = nil;
							end
						end
					else
						--ExPack:dPrint("Attachments_Update; tframe; not created; "..index.."; "..itemID)
					end
					if (button.ExPack) then
						if profile.visualShowCaption and not bCleanUp then
							--ExPack:dPrint("InboxFrame_Update; button; ShowCaption; "..index.."; "..itemID)
							button.ExPack:Show()
						else
							--ExPack:dPrint("InboxFrame_Update; button; HideCaption; "..index.."; "..itemID)
							button.ExPack:Hide()
							if (bCleanUp) then
								button.ExPack = nil;
							end
						end
					else
						--ExPack:dPrint("Attachments_Update; button.ExPack; not created; "..index.."; "..itemID)
					end
				end
			end
		end
		index = index + 1;
	end

end

function ExPack_OpenMailFrame_UpdateButtonPositions()

	ExPack:dPrint( "OpenMailFrame_UpdateButtonPositions; " );

	local profile = ExPack.profile

	local bCleanUp = false;
	if not profile.filtersShowOpenMailAttachments then
		--ExPack.logger:Warn( "OpenMailFrame_UpdateButtonPositions; filtersShowOpenMailAttachments; false" );
		bCleanUp = true;
	end

	for i=1, ATTACHMENTS_MAX_RECEIVE do
		local attachmentButton = OpenMailFrame.OpenMailAttachments[i];
		if HasInboxItem(InboxFrame.openMailID, i) then

			local name, itemID, itemTexture, count, quality, canUse = GetInboxItem(InboxFrame.openMailID, i);
			if name then
				ExPack.logger:Debug( "attachmentButton; "..i.."; "..itemID.."; "..count.."; "..name );
			end

			local itemLink = GetInboxItemLink(InboxFrame.openMailID, i);
			if (itemLink) then
				--ExPack:dPrint( "attachmentButton; "..i.."; itemLink" );
			end

			if (itemID) then
				local tframe = ExPack_GetExPackFrame(attachmentButton, itemID, itemLink, 0, 0, bCleanUp);

				if (tframe) then
					if (profile.visualShowCrown and not bCleanUp) then
						--ExPack:dPrint("OpenMailFrame_UpdateButtonPositions; tframe; ShowCrown; "..i.."; "..itemID)
						tframe:Show()
					else
						--ExPack:dPrint("OpenMailFrame_UpdateButtonPositions; tframe; HideCrown; "..i)
						tframe:Hide()
						if (bCleanUp) then
							tframe = nil;
						end
					end
				end
				if (attachmentButton.ExPack) then
					if profile.visualShowCaption and not bCleanUp then
						--ExPack:dPrint("OpenMailFrame_UpdateButtonPositions; button; ShowCaption; "..i.."; "..itemID)
						attachmentButton.ExPack:Show()
					else
						--ExPack:dPrint("OpenMailFrame_UpdateButtonPositions; button; HideCaption; "..i)
						attachmentButton.ExPack:Hide()
						if (bCleanUp) then
							attachmentButton.ExPack = nil;
						end
					end
				end
			end

		end
	end

end

function ExPack_SendMailFrame_Update()
	ExPack.logger:Debug("ExPack_SendMailFrame_Update")

	local profile = ExPack.profile
	local bCleanUp = false;
	if not profile.filtersShowSendMailAttachments then
		--ExPack.logger:Warn( "ExPack_SendMailFrame_Update; filtersShowSendMailAttachments; false" );
		bCleanUp = true;
	end

	-- Update the item(s) being sent
	local itemCount = 0;
	for i=1, ATTACHMENTS_MAX_SEND do
		local attachmentButton = SendMailFrame.SendMailAttachments[i];

		if HasSendMailItem(i) then
			itemCount = itemCount + 1;

			local itemName, itemID, itemTexture, stackCount, quality = GetSendMailItem(i);
			if itemName then
				ExPack.logger:Trace( "attachmentButton; "..i.."; "..itemID.."; "..itemCount.."; "..itemName );
			end

			local itemLink = GetSendMailItemLink(i);
			if (itemLink) then
				ExPack.logger:Trace( "attachmentButton; "..i.."; itemLink" );
			end

			ExPack_GetExPackFrame(attachmentButton, itemID, itemLink, 0, 0, bCleanUp);
			ExPack_UpdateExPackFrame(attachmentButton, bCleanUp);
		else
			-- cleanup
			if (attachmentButton.ExPackIcon) then
					attachmentButton.ExPackIcon:Hide()
					attachmentButton.ExPackIcon = nil;
			end
			if (attachmentButton.ExPack) then
					attachmentButton.ExPack:Hide()
					attachmentButton.ExPack = nil;
			end
		end -- HasSendMailItem
	end -- for/loop

end

function ExPack:MAIL_INBOX_UPDATE(event)
	self.logger:Trace(event)
end

function ExPack:MAIL_SEND_INFO_UPDATE(event)
	self.logger:Trace(event)
end

--[-[- ATTACHMENT FEATURE END -]-]--

--[-[- PlugIns Mechanism BEGIN -]-]--

function ExPack:LoadPlugins()
	-- todo
	if IsAddOnLoaded("Bagnon") or IsAddOnLoaded("Combuctor") then
		--self.plugin = self:GetModule("PluginBagnonCombuctor")
	elseif IsAddOnLoaded("ArkInventory") then
		--self.plugin = self:GetModule("PluginArkInventory")
	elseif IsAddOnLoaded("LiteBag") then
		--self.plugin = self:GetModule("PluginLiteBag")
	elseif IsAddOnLoaded("AdiBags") then
		--self.plugin = self:GetModule("PluginAdiBags")
	elseif IsAddOnLoaded("ElvUI") then
		self.plugin = self:GetModule("PluginElvUI")
	elseif IsAddOnLoaded("PluginDefault") then
		self.plugin = self:GetModule("PluginDefault")
	end

	if (self.plugin) then
		self.plugin:Init(self.repo, self.config)
		self.logger:Debug(self.plugin:GetName(), "loaded")
		self.PluginsMode = true
	else
		self.logger:Debug("No plugin loaded")
		self.PluginsMode = false
	end
end

function ExPack:UpdateSlots(bagFrame)
	if (self.plugin) then
		if (bagFrame) then
			local bagFrameName = bagFrame:GetName()
			self.logger:Debug("UpdateSlots", bagFrameName)
		end

		for bagID, slotFrames in pairs(self.utils:GetSlotsByBagID(self.plugin)) do
			for _idx, slotFrame in pairs(slotFrames) do
				self:UpdateSlot(bagID, slotFrame)
			end
		end
	end
end

function ExPack:UpdateSlot(bagID, slotFrame)
	local profile = ExPack.profile
	if not profile.Enabled then
		self.logger:Debug("UpdateSlot; generalEnabled; false")
		return
	end

	if not slotFrame then
		self.logger:Debug("UpdateSlot; not slotFrame; ", bagID)
		return
	end
	local bCleanUp = false;
	--self.logger:Debug("UpdateSlot; bagID", bagID, slotFrame:GetName())

	local slotID = slotFrame:GetID()
	--self.logger:Debug("UpdateSlot; slotID", slotID)

	local item = Item:CreateFromBagAndSlot(bagID, slotID)
	if (item) then
		local itemID = item:GetItemID()
		--self.logger:Debug("UpdateSlot; item", item)
		if item:IsItemEmpty() then
			--self.logger:Debug("UpdateSlot; IsItemEmpty")
		end
		local itemLink = item:GetItemLink()
		if (itemLink) then
			--self.logger:Debug("UpdateSlot; itemLink", itemLink)
			if itemID == nil then
				itemID = GetItemInfoFromHyperlink(itemLink)
			end
		else
			--self.logger:Debug("UpdateSlot; no itemLink")
		end

		local tframe = ExPack_GetExPackFrame(slotFrame, itemID, itemLink, 0, 0, bCleanUp)
		if (itemID) then
			--self.logger:Debug("UpdateSlot; itemID", itemID, bagID, slotID)
			if (tframe) then
				if profile.visualShowCrown then
					tframe:Show();
				else
					tframe:Hide();
				end
				if slotFrame.ExPack then
					if profile.visualShowCaption then
						slotFrame.ExPack:Show();
					else
						slotFrame.ExPack:Hide();
					end
				end
			end
		else
			--self.logger:Debug("UpdateSlot; no itemID")
			if (tframe) then
				tframe:Hide();
				local tframeName = tframe:GetName()
				_G[tframeName] = nil;
			end
			if slotFrame.ExPack then
				slotFrame.ExPack:Hide();
				slotFrame.ExPack = nil;
			end
		end

		--ExPack:vardump(item)
	else
		self.logger:Debug("UpdateSlot; no item")
	end

	--if self.iCount > 0 then
	--	self.iCount = self.iCount - 1
	--end
end

--[-[- PlugIns Mechanism END -]-]--
