local ExPack = LibStub("AceAddon-3.0"):GetAddon("ExPack")
local Logger = ExPack:NewModule("Logger")

local logLevels = {
	TRACE = 0,
	DEBUG = 1,
	INFO = 2,
	WARN = 3,
	ERROR = 4
}

local bLoggerInit = false;

function Logger:Init(config)
	self.config = config
	--self:Print(self:Cyan("[DEBUG]"), "Logger:Init")
	bLoggerInit = true;
end

function Logger:GetLogLevel()
	if bLoggerInit then
		return self.config:GetLogLevel()
	else
		return logLevels.DEBUG;
	end
end

function Logger:Trace(...)
	self:MaybeLog("TRACE", self:Cyan("[TRACE]"), ...)
end


function Logger:Debug(...)
	local bShow = self.config:DebugTracing();
	if bShow then
		ExPack:Print(self:Cyan("[DEBUG]"), ...)
	else
		self:MaybeLog("DEBUG", self:Cyan("[DEBUG]"), ...)
	end
end

function Logger:Info(...)
	self:MaybeLog("INFO", self:Yellow("[INFO]"), ...)
end

function Logger:Warn(...)
	self:MaybeLog("WARN", self:Red("[WARN]"), ...)
end

function Logger:Error(...)
	local bShow = self.config:ErrorReporting();
	if bShow then
		ExPack:Print(self:Red("[ERROR]"), ...)
	else
		self:MaybeLog("ERROR", ...)
	end
end

function Logger:Print(...)
	ExPack:Print(...)
end

function Logger:MaybeLog(level, ...)
	local logLevel = self:GetLogLevel()
	if (logLevels[logLevel] <= logLevels[level]) then
		ExPack:Print(...)
	end
end

function Logger:Yellow(...)
	return "|cFFFFFF00" .. ... .. "|r"
end

function Logger:Cyan(...)
	return "|cFF00FFFF" .. ... .. "|r"
end

function Logger:Red(...)
	return "|cFFFF0000" .. ... .. "|r"
end

function Logger:BoolToStr(v)
	if v then
		return "true"
	else
		return "false"
	end
end
