local ExPack = LibStub("AceAddon-3.0"):GetAddon("ExPack")
local PluginDefault = ExPack:NewModule("PluginDefault")

local GetContainerNumSlots
if C_Container then
	GetContainerNumSlots = C_Container.GetContainerNumSlots
else
	GetContainerNumSlots = _G.GetContainerNumSlots
end

local lastBagIndex = nil

local function handleBagUpdate(bagIndex)
	lastBagIndex = bagIndex
	local bagID = bagIndex + 1
	local bagFrame = _G["ContainerFrame" .. bagID]

	if bagFrame then
		-- event fires twice in Classic with -2 bagIndex for the backpack in classic for some reason
		local bagName = bagFrame:GetName()
		local bagSize = GetContainerNumSlots(bagIndex)
		
		for itemIndex = 1, bagSize, 1 do
			local slotIndex = bagSize - itemIndex + 1
			local slotFrame = _G[bagName .. "Item" .. slotIndex]
			ExPack:UpdateSlot(bagIndex, slotFrame)
		end
	end
end

local function onEvent(self, event, ...)
	if event == "BAG_UPDATE" then
		handleBagUpdate(...)
	elseif event == "BAG_UPDATE_DELAYED" then
		handleBagUpdate(lastBagIndex)
		lastBagIndex = nil
	end
end

function PluginDefault:Init()
	if _G.ContainerFrame_OnShow then
		hooksecurefunc("ContainerFrame_OnShow", function(...)
			ExPack:UpdateSlots(...)
		end)
	end

	self.frame = CreateFrame("Frame")
	self.frame:RegisterEvent("BAG_UPDATE")
	self.frame:RegisterEvent("BAG_UPDATE_DELAYED")
	self.frame:SetScript("OnEvent", onEvent)
end

function PluginDefault:GetSlotFrame(bagID, slotIndex)
	local containerFrameIndex = bagID + 1
	local itemIndex = GetContainerNumSlots(bagID) - (slotIndex - 1)

	return _G["ContainerFrame" .. containerFrameIndex .. "Item" .. itemIndex]
end
